@@ -27,7 +27,7 @@ import unicodedata
 import zlib
 
 def getVersion() -> str:
-    return "1.5.2"
+    return "1.5.6"
 
 
 #########################
@@ -114,6 +114,7 @@ class StatusBar:
     def update(self, msg: str) -> None:
         if self.display:
             self.progress += 1
+            msg = msg.encode("utf-8", "surrogateescape").decode("utf-8", "replace")
             if self.total == -1:
                 progress_str = str(self.progress) + ": "
                 self.msg_len = self.char_display - len(progress_str) - len(self.title_str)
@@ -179,7 +180,7 @@ class ConfigObject:
         self.verbose = True
         # load config
         for key in config:
-            if config[key] is not None:
+            if config[key] is not None and hasattr(self, key):
                 self.__setattr__(key, config[key])
         # normalize paths (these should be relative, not absolute!)
         self.archive_dir = os.path.normpath(self.archive_dir)
@@ -196,11 +197,14 @@ class ConfigObject:
 class DirInfo:
     def __init__(self, directory_root_path: str, compare_mode: str,  config_dir: str, ignored_toplevel_folders: list = [], gui: bool = False, force_posix_path_sep: bool = False):
         """For scanning directories, tracking files and changes, meant only for internal use by BackupManager"""
-        self.file_dicts = {}
-        self.loaded_dicts = {}
-        self.loaded_diffs = {}
-        self.missing_files = {}
-        self.crc_errors_detected = {}
+        # File dictionaries, keys are paths relative to directory_root_path, values are dictionaries of file attributes
+        self.dict_current = {}
+        self.dict_prev = {}
+        self.dict_modified = {}
+        self.dict_missing = {}
+        self.dict_new = {}
+        self.dict_crc_errors = {}
+        # Init other variables
         self.dir = directory_root_path
         self.compare_mode = compare_mode
         self.config_dir = config_dir
@@ -208,26 +212,15 @@ class DirInfo:
         self.gui = gui
         self.force_posix_path_sep = force_posix_path_sep
 
-    def getDirDict(self) -> dict:
-        return self.file_dicts
-
-    def getLoadedDicts(self) -> dict:
-        return self.loaded_dicts
-
-    def getLoadedDiffs(self) -> dict:
-        return self.loaded_diffs
-
-    def getMissingFiles(self) -> dict:
-        return self.missing_files
-
-    def getCrcErrorsDetected(self) -> dict:
-        return self.crc_errors_detected
+    def getDicts(self) -> tuple:
+        """Returns tuple of dictionaries: current, prev, new, modified, missing, crc_errors"""
+        return self.dict_current, self.dict_prev, self.dict_new, self.dict_modified, self.dict_missing, self.dict_crc_errors
 
     def saveJson(self, db_name: str = "database.json") -> None:
-        writeJson(os.path.join(self.dir, self.config_dir, db_name), self.file_dicts, sort_keys=True)
+        writeJson(os.path.join(self.dir, self.config_dir, db_name), self.dict_current, sort_keys=True)
 
     def loadJson(self) -> None:
-        self.loaded_dicts = readJson(os.path.join(self.dir, self.config_dir, "database.json"))
+        self.dict_prev = readJson(os.path.join(self.dir, self.config_dir, "database.json"))
 
     def verifyCrcOnCopy(self, source_root: str, dest_root: str, source_file: str, dest_file: str, secondInfo: 'DirInfo') -> None:
         if self.dir == source_root and secondInfo.dir == dest_root:
@@ -239,37 +232,37 @@ class DirInfo:
 
     def updateDictOnCopy(self, source_root: str, dest_root: str, source_file: str, dest_file: str, secondInfo: 'DirInfo') -> None:
         if self.dir == source_root and secondInfo.dir == dest_root:
-            secondInfo.file_dicts[dest_file] = self.file_dicts[source_file].copy()
+            secondInfo.dict_current[dest_file] = self.dict_current[source_file].copy()
         elif self.dir == dest_root and secondInfo.dir == source_root:
-            self.file_dicts[dest_file] = secondInfo.file_dicts[source_file].copy()
+            self.dict_current[dest_file] = secondInfo.dict_current[source_file].copy()
         else:
             raise Exception("Update Database Error")
 
     def updateDictOnMove(self, source_root: str, dest_root: str, source_file: str, dest_file: str, secondInfo: 'DirInfo') -> None:
         if source_root == dest_root == self.dir:
-            self.file_dicts[dest_file] = self.file_dicts.pop(source_file)
+            self.dict_current[dest_file] = self.dict_current.pop(source_file)
         elif source_root == dest_root == secondInfo.dir:
-            secondInfo.file_dicts[dest_file] = secondInfo.file_dicts.pop(source_file)
+            secondInfo.dict_current[dest_file] = secondInfo.dict_current.pop(source_file)
         elif source_root == self.dir and dest_root != secondInfo.dir:
-            _ = self.file_dicts.pop(source_file)
+            _ = self.dict_current.pop(source_file)
         elif source_root == secondInfo.dir and dest_root != self.dir:
-            _ = secondInfo.file_dicts.pop(source_file)
+            _ = secondInfo.dict_current.pop(source_file)
         else:
             raise Exception("Update Database Error")
 
     def updateDictOnRemove(self, root_path: str, file_relative_path: str, secondInfo: 'DirInfo') -> None:
         if root_path == self.dir:
-            _ = self.file_dicts.pop(file_relative_path)
+            _ = self.dict_current.pop(file_relative_path)
         elif root_path == secondInfo.dir:
-            _ = secondInfo.file_dicts.pop(file_relative_path)
+            _ = secondInfo.dict_current.pop(file_relative_path)
         else:
             raise Exception("Update Database Error")
 
     def getCrc(self, relative_path: str, recalc: bool = False) -> str:
-        if recalc or "crc" not in self.file_dicts[relative_path]:
+        if recalc or "crc" not in self.dict_current[relative_path]:
             full_path = os.path.join(self.dir, relative_path)
-            self.file_dicts[relative_path]["crc"] = self.calcCrc(full_path)
-        return self.file_dicts[relative_path]["crc"]
+            self.dict_current[relative_path]["crc"] = self.calcCrc(full_path)
+        return self.dict_current[relative_path]["crc"]
 
     def calcCrc(self, file_path: str, prev: int = 0) -> str:
         with open(file_path, "rb") as f:
@@ -306,29 +299,29 @@ class DirInfo:
         return False
 
     def fileMatch(self, f1: str, f2: str, secondInfo: 'DirInfo', compare_mode: str, move_check: bool = False) -> bool:
-        if self.file_dicts[f1]["size"] == secondInfo.file_dicts[f2]["size"]:
-            if self.timeMatch(self.file_dicts[f1]["mtime"], secondInfo.file_dicts[f2]["mtime"], False, [3600, 3601, 3602]):
+        if self.dict_current[f1]["size"] == secondInfo.dict_current[f2]["size"]:
+            if self.timeMatch(self.dict_current[f1]["mtime"], secondInfo.dict_current[f2]["mtime"], False, [3600, 3601, 3602]):
                 # these are the 'unchanged files (probably)' from both sides, crc should be up to date from the scan if using CRC mode
-                if compare_mode == "crc" and self.file_dicts[f1]["crc"] != secondInfo.file_dicts[f2]["crc"]:
+                if compare_mode == "crc" and self.dict_current[f1]["crc"] != secondInfo.dict_current[f2]["crc"]:
                     # size and date match, but crc does not, probably corrupted
-                    if f1 not in self.crc_errors_detected and f2 not in secondInfo.crc_errors_detected:
+                    if f1 not in self.dict_crc_errors and f2 not in secondInfo.dict_crc_errors:
                         # log error since it wasn't already detected during scan, that implies neither file has a past record
-                        self.crc_errors_detected[f1] = None
-                        secondInfo.crc_errors_detected[f2] = None
+                        self.dict_crc_errors[f1] = None
+                        secondInfo.dict_crc_errors[f2] = None
                     return False
                 # detect mismatched crc values across both sides (usually if corruption happened before crc database was created)
-                if compare_mode == "attr+" and self.file_dicts[f1]["crc"] != secondInfo.file_dicts[f2]["crc"]:
+                if compare_mode == "attr+" and self.dict_current[f1]["crc"] != secondInfo.dict_current[f2]["crc"]:
                     if move_check:
                         # it may be corrupted or coincidence, just won't flag these files as matching
                         return False
-                    self.crc_errors_detected[f1] = self.file_dicts[f1]
-                    secondInfo.crc_errors_detected[f2] = secondInfo.file_dicts[f2]
+                    self.dict_crc_errors[f1] = self.dict_current[f1]
+                    secondInfo.dict_crc_errors[f2] = secondInfo.dict_current[f2]
                 return True
         return False
 
     def scanDir(self, stdout_status_bar: bool) -> None:
         # init
-        if os.path.isdir(self.dir) and self.file_dicts == {}:
+        if os.path.isdir(self.dir) and self.dict_current == {}:
             total = sum(len(f) for r, d, f in os.walk(self.dir))
             scan_status = StatusBar("Scanning", total, stdout_status_bar, gui=self.gui)
             # will never enable followlinks, adds too many possible issues and complexity in handling them
@@ -345,7 +338,7 @@ class DirInfo:
                         relative_path = os.path.relpath(full_path, self.dir)
                         if self.force_posix_path_sep:
                             relative_path = relative_path.replace(os.path.sep, "/")
-                        self.file_dicts[relative_path] = {"size": 0, "mtime": 0, "crc": "0", "dir": True}
+                        self.dict_current[relative_path] = {"size": 0, "mtime": 0, "crc": "0", "dir": True}
                 # scan files
                 for file_name in file_list:
                     full_path = os.path.join(dir_path, file_name)
@@ -357,54 +350,55 @@ class DirInfo:
                     size = os.path.getsize(full_path)
                     mtime = os.path.getmtime(full_path)
                     # check and set database dictionaries
-                    if relative_path in self.loaded_dicts:
-                        if (self.loaded_dicts[relative_path]["size"] == size and
-                            self.timeMatch(self.loaded_dicts[relative_path]["mtime"], mtime, True)):
+                    if relative_path in self.dict_prev:
+                        if (self.dict_prev[relative_path]["size"] == size and
+                            self.timeMatch(self.dict_prev[relative_path]["mtime"], mtime, True)):
                             # unchanged file (probably)
-                            self.file_dicts[relative_path] = self.loaded_dicts[relative_path].copy()
+                            self.dict_current[relative_path] = self.dict_prev[relative_path].copy()
                         else:
                             # changed file
-                            self.file_dicts[relative_path] = {"size": size, "mtime": mtime}
-                            self.loaded_diffs[relative_path] = self.loaded_dicts[relative_path]
+                            self.dict_current[relative_path] = {"size": size, "mtime": mtime}
+                            self.dict_modified[relative_path] = self.dict_prev[relative_path]
                     else:
                         # new file
-                        self.file_dicts[relative_path] = {"size": size, "mtime": mtime}
+                        self.dict_current[relative_path] = {"size": size, "mtime": mtime}
+                        self.dict_new[relative_path] = self.dict_current[relative_path]
                     if self.compare_mode == "crc":
                         # calculate CRC for all files (simpler code and potential warning sign for disk issues)
-                        self.file_dicts[relative_path]["crc"] = self.calcCrc(full_path)
-                        if (relative_path in self.loaded_dicts and
-                            "crc" in self.loaded_dicts[relative_path] and
-                            self.loaded_dicts[relative_path]["crc"] != self.file_dicts[relative_path]["crc"] and
-                            self.loaded_dicts[relative_path]["size"] == size and
-                            self.timeMatch(self.loaded_dicts[relative_path]["mtime"], mtime, False, [3600, 3601, 3602])):
+                        self.dict_current[relative_path]["crc"] = self.calcCrc(full_path)
+                        if (relative_path in self.dict_prev and
+                            "crc" in self.dict_prev[relative_path] and
+                            self.dict_prev[relative_path]["crc"] != self.dict_current[relative_path]["crc"] and
+                            self.dict_prev[relative_path]["size"] == size and
+                            self.timeMatch(self.dict_prev[relative_path]["mtime"], mtime, False, [3600, 3601, 3602])):
                             # corrupted file (probably, changed crc, unchanged size and mtime)
-                            self.crc_errors_detected[relative_path] = self.loaded_dicts[relative_path]
-                    elif self.compare_mode == "attr+" and "crc" not in self.file_dicts[relative_path]:
+                            self.dict_crc_errors[relative_path] = self.dict_prev[relative_path]
+                    elif self.compare_mode == "attr+" and "crc" not in self.dict_current[relative_path]:
                         # save time by only calculating crc for new and changed files (by attributes) so we can check for corruption later (and possibly preexisting)
-                        if (relative_path in self.loaded_dicts and
-                            "crc" in self.loaded_dicts[relative_path] and
-                            self.loaded_dicts[relative_path]["size"] == size and
-                            self.timeMatch(self.loaded_dicts[relative_path]["mtime"], mtime, False, [3600, 3601, 3602])):
+                        if (relative_path in self.dict_prev and
+                            "crc" in self.dict_prev[relative_path] and
+                            self.dict_prev[relative_path]["size"] == size and
+                            self.timeMatch(self.dict_prev[relative_path]["mtime"], mtime, False, [3600, 3601, 3602])):
                             # attributes match, preserve old crc
-                            self.file_dicts[relative_path]["crc"] = self.loaded_dicts[relative_path]["crc"]
+                            self.dict_current[relative_path]["crc"] = self.dict_prev[relative_path]["crc"]
                         else:
-                            self.file_dicts[relative_path]["crc"] = self.calcCrc(full_path)
+                            self.dict_current[relative_path]["crc"] = self.calcCrc(full_path)
             scan_status.endProgress()
             # check for missing (or moved) files
-            for relative_path in (set(self.loaded_dicts) - set(self.file_dicts)):
-                if "dir" not in self.loaded_dicts[relative_path]:
+            for relative_path in (set(self.dict_prev) - set(self.dict_current)):
+                if "dir" not in self.dict_prev[relative_path]:
                     if not self.pathMatch(relative_path, self.ignored_toplevel_folders):
-                        self.missing_files[relative_path] = self.loaded_dicts[relative_path]
+                        self.dict_missing[relative_path] = self.dict_prev[relative_path]
 
     def dirCompare(self, secondInfo: 'DirInfo', no_moves: bool = False, filter_include_list: typing.Union[list, None] = None, filter_exclude_list: typing.Union[list, None] = None) -> tuple:
-        # init variables
-        file_list = set(self.file_dicts)
-        second_list = set(secondInfo.getDirDict())
+        # init variables, todo: if secondInfo == self, second_list = set(secondInfo.dict_prev), iff compare logic below still works (to check moved for scan_only and reuse code)
+        file_list = set(self.dict_current)
+        second_list = set(secondInfo.dict_current)
         if self.compare_mode == secondInfo.compare_mode:
             compare_mode = self.compare_mode
         else:
             raise Exception("Inconsistent compare mode between directories")
-        # apply filters
+        # apply filters, todo: compile filters in init and execute during scan
         if type(filter_include_list) == list:
             for i in range(len(filter_include_list)):
                 if type(filter_include_list[i]) == str:
@@ -428,15 +422,15 @@ class DirInfo:
         moved = []
         if not no_moves:
             for f1 in reversed(self_only):
-                if "dir" not in self.file_dicts[f1]:
+                if "dir" not in self.dict_current[f1]:
                     for f2 in reversed(second_only):
-                        if "dir" not in secondInfo.file_dicts[f2]:
+                        if "dir" not in secondInfo.dict_current[f2]:
                             if self.fileMatch(f1, f2, secondInfo, compare_mode, True):
                                 moved.append({"source": f1, "dest": f2})
                                 self_only.remove(f1)
                                 second_only.remove(f2)
-                                _ = secondInfo.missing_files.pop(f1, 1)
-                                _ = self.missing_files.pop(f2, 1)
+                                _ = secondInfo.dict_missing.pop(f1, 1)
+                                _ = self.dict_missing.pop(f2, 1)
                                 break
             moved.reverse()
         return self_only, second_only, changed, moved
@@ -585,6 +579,13 @@ class BackupManager:
         else:
             return "{:<10}".format("%s B" %(size))
 
+    def prettyAttr(self, attr: dict) -> str:
+        string = "{'size': %s, 'mtime': %s" %(attr["size"], attr["mtime"])
+        if "crc" in attr: string += ", 'crc': '%s'" %(attr["crc"])
+        if "dir" in attr: string += ", 'dir': %s" %(attr["dir"])
+        string += "}"
+        return string
+
     ########################
     ### Printing methods ###
     ########################
@@ -592,14 +593,14 @@ class BackupManager:
     def colourPrint(self, msg: str, colour: str) -> None:
         if self.config.verbose:
             if colour == "NONE":
-                print(msg)
+                print(self.replaceSurrogates(msg))
             else:
                 print(self.colourString(msg, colour))
 
     def printFileInfo(self, header: str, f: str, d: dict, sub_header: str = "", skip_info: bool = False) -> None:
         header, sub_header = getString(header), getString(sub_header)
         if f in d and d[f] is not None:
-            self.log.append([header.strip(), sub_header.strip(), f] + [str(d[f])])
+            self.log.append([header.strip(), sub_header.strip(), f] + [self.prettyAttr(d[f])])
             missing = False
         else:
             self.log.append([header.strip(), sub_header.strip(), f] + [getString("Missing")])
@@ -625,6 +626,11 @@ class BackupManager:
         for f in l:
             self.printFileInfo("File: ", f, d)
 
+    def printScanChanges(self, l: list, d1: dict, d2: dict) -> None:
+        for f in l:
+            self.printFileInfo("File: ", f, d1, "    New")
+            self.printFileInfo("", f, d2, "    Old")
+
     def printChangedFiles(self, l: list, d1: dict, d2: dict) -> None:
         for f in l:
             self.printFileInfo("File: ", f, d1, " Source")
@@ -781,73 +787,39 @@ class BackupManager:
                 break
         copy_status.endProgress()
 
-    ######################################
-    ### Main backup/mirror/sync method ###
-    ######################################
+    #########################################
+    ### Helper functions used by backup() ###
+    #########################################
 
-    def backup(self):
-        """Main method, use this to run your job no matter the configuration"""
-        if self.config.dry_run:
-            print(self.colourString(getString("Dry Run"), "HEADER"))
-        # init dir scanning and load previous scan data if available
-        self.source = DirInfo(self.config.source, self.config.compare_mode, self.config.config_dir,
-                              [self.config.archive_dir, self.config.log_dir, self.config.trash_dir],
-                              self.gui, self.config.force_posix_path_sep)
-        self.dest = DirInfo(self.config.dest, self.config.compare_mode, self.config.config_dir,
-                            [self.config.archive_dir, self.config.log_dir, self.config.trash_dir],
-                            self.gui, self.config.force_posix_path_sep)
-        database_load_success = False
-        self.source.loadJson()
-        self.dest.loadJson()
-        if self.source.loaded_dicts != {} or self.dest.loaded_dicts != {}:
-            database_load_success = True
-        # scan directories (also calculates CRC if enabled) (didn't parallelize scans to prevent excess vibration of adjacent consumer grade disks and keep status bars simple)
-        self.colourPrint(getString("Scanning files on source:\n%s") %(self.config.source), "OKBLUE")
-        self.source.scanDir(self.config.stdout_status_bar)
-        if self.config.source != self.config.dest:
-            self.colourPrint(getString("Scanning files on destination:\n%s") %(self.config.dest), "OKBLUE")
-            self.dest.scanDir(self.config.stdout_status_bar)
-        else:
-            self.dest = self.source
-        # compare directories (should be relatively fast, all the read operations are done during scan)
-        if not self.config.scan_only:
-            self.colourPrint(getString("Comparing directories..."), "OKBLUE")
-            source_only, dest_only, changed, moved = self.source.dirCompare(self.dest,
-                                                                            self.config.nomoves,
-                                                                            self.config.filter_include_list,
-                                                                            self.config.filter_exclude_list)
+    def _databaseAndCorruptionCheck(self, dest_database_load_success: bool) -> bool:
         # get databases
-        source_dict = self.source.getDirDict()
-        source_diffs = self.source.getLoadedDiffs()
-        source_missing = self.source.getMissingFiles()
-        source_loaded_db = self.source.getLoadedDicts()
-        source_crc_errors = self.source.getCrcErrorsDetected()
-        dest_dict = self.dest.getDirDict()
-        dest_diffs = self.dest.getLoadedDiffs()
-        dest_missing = self.dest.getMissingFiles()
-        dest_loaded_db = self.dest.getLoadedDicts()
-        dest_crc_errors = self.dest.getCrcErrorsDetected()
+        source_dict, source_prev, source_new, source_modified, source_missing, source_crc_errors = self.source.getDicts()
+        dest_dict, dest_prev, dest_new, dest_modified, dest_missing, dest_crc_errors = self.dest.getDicts()
         # print database conflicts, including both collisions from files being modified independently on both sides and unexpected missing files
         # note: this only notifies the user so they can intervene, it does not handle them in any special way, treating them as regular file changes
         # it can also be triggered by time zone or dst changes, lower file system mod time precision, and corruption if using CRCs (handled next)
         abort_run = False
-        if database_load_success:
+        if dest_database_load_success and self.config.source != self.config.dest:
             self.log.append([getString("### DATABASE CONFLICTS ###")])
             if self.config.main_mode == "sync":
-                sync_conflicts = sorted(list(set(source_diffs) & set(dest_diffs)))
-                sync_conflicts += sorted(list(set(source_missing) | set(dest_missing)))
+                sync_conflicts = sorted(list(set(source_modified) & set(dest_modified))) # modified on both sides
+                sync_conflicts += sorted(list(set(source_missing) | set(dest_missing))) # deleted from either or both sides
+                sync_conflicts += sorted(list(set(source_new) & set(dest_new))) # new on both sides
+                # sync_conflicts += sorted(list(filter(lambda f: source_new[f] != dest_new[f], set(source_new) & set(dest_new)))) # new and different on both sides
                 if len(sync_conflicts) >= 1:
                     print(self.colourString(getString("WARNING: found files modified in both source and destination since last scan"), "WARNING"))
                     abort_run = True
                 print(self.colourString(getString("Sync Database Conflicts: %s") %(len(sync_conflicts)), "HEADER"))
-                self.printSyncDbConflicts(sync_conflicts, source_dict, dest_dict, source_loaded_db, dest_loaded_db)
+                self.printSyncDbConflicts(sync_conflicts, source_dict, dest_dict, source_prev, dest_prev)
             else:
-                dest_conflicts = sorted(list(set(dest_diffs) | set(dest_missing)))
+                dest_conflicts = sorted(list(set(dest_modified)))
+                dest_conflicts += sorted(list(set(dest_missing)))
+                dest_conflicts += sorted(list(set(dest_new)))
                 if len(dest_conflicts) >= 1:
                     print(self.colourString(getString("WARNING: found files modified in the destination since last scan"), "WARNING"))
                     abort_run = True
                 print(self.colourString(getString("Destination Database Conflicts: %s") %(len(dest_conflicts)), "HEADER"))
-                self.printDbConflicts(dest_conflicts, dest_dict, dest_loaded_db)
+                self.printDbConflicts(dest_conflicts, dest_dict, dest_prev)
         # print database conflicts concerning CRCs if available, as well as CRC conflicts between source and dest if attributes otherwise match
         if len(source_crc_errors) > 0 or len(dest_crc_errors) > 0:
             self.log.append([getString("### CRC ERRORS DETECTED ###")])
@@ -856,20 +828,43 @@ class BackupManager:
             if self.config.compare_mode == "crc":
                 crc_errors_detected = sorted(list(set(source_crc_errors) | set(dest_crc_errors)))
                 print(self.colourString(getString("CRC Errors Detected: %s") %(len(crc_errors_detected)), "HEADER"))
-                self.printSyncDbConflicts(crc_errors_detected, source_dict, dest_dict, source_loaded_db, dest_loaded_db)
+                self.printSyncDbConflicts(crc_errors_detected, source_dict, dest_dict, source_prev, dest_prev)
             elif self.config.compare_mode == "attr+":
                 if set(source_crc_errors) != set(dest_crc_errors):
                     raise Exception("Inconsistent CRC error detection between source and dest")
                 print(self.colourString(getString("CRC Errors Detected: %s") %(len(source_crc_errors)), "HEADER"))
                 self.printChangedFiles(sorted(list(source_crc_errors)), source_crc_errors, dest_crc_errors)
-        if self.config.quit_on_db_conflict and abort_run:
-            return self.abortRun()
-        # end scan
-        if self.config.scan_only:
-            self.log.append([getString("### SCAN COMPLETED ###")])
-            self.writeLog("database.json")
-            print(self.colourString(getString("Completed!"), "OKGREEN"))
-            return 0
+        return abort_run
+
+    def _printAndLogScanOnlyDiffSummary(self) -> None:
+        # get databases
+        source_dict, source_prev, source_new, source_modified, source_missing, _ = self.source.getDicts()
+        dest_dict, dest_prev, dest_new, dest_modified, dest_missing, _ = self.dest.getDicts()
+        # print differences
+        print(self.colourString(getString("Source New Files: %s") %(len(source_new)), "HEADER"))
+        self.log.append([getString("### SOURCE NEW FILES ###")])
+        self.printFiles(sorted(list(source_new)), source_dict)
+        print(self.colourString(getString("Source Missing Files: %s") %(len(source_missing)), "HEADER"))
+        self.log.append([getString("### SOURCE MISSING FILES ###")])
+        self.printFiles(sorted(list(source_missing)), source_prev)
+        print(self.colourString(getString("Source Changed Files: %s") %(len(source_modified)), "HEADER"))
+        self.log.append([getString("### SOURCE CHANGED FILES ###")])
+        self.printScanChanges(sorted(list(source_modified)), source_dict, source_prev)
+        if self.config.source != self.config.dest:
+            print(self.colourString(getString("Destination New Files: %s") %(len(dest_new)), "HEADER"))
+            self.log.append([getString("### DESTINATION NEW FILES ###")])
+            self.printFiles(sorted(list(dest_new)), dest_dict)
+            print(self.colourString(getString("Destination Missing Files: %s") %(len(dest_missing)), "HEADER"))
+            self.log.append([getString("### DESTINATION MISSING FILES ###")])
+            self.printFiles(sorted(list(dest_missing)), dest_prev)
+            print(self.colourString(getString("Destination Changed Files: %s") %(len(dest_modified)), "HEADER"))
+            self.log.append([getString("### DESTINATION CHANGED FILES ###")])
+            self.printScanChanges(sorted(list(dest_modified)), dest_dict, dest_prev)
+
+    def _printAndLogCompareDiffSummary(self, source_only: list, dest_only: list, changed: list, moved: list) -> None:
+        # get databases
+        source_dict, _, _, _, _, _ = self.source.getDicts()
+        dest_dict, _, _, _, _, _ = self.dest.getDicts()
         # prepare diff messages
         if self.config.noarchive:
             archive_msg = getString("delete")
@@ -889,10 +884,6 @@ class BackupManager:
             change_msg = getString("(%s older and copy newer)" %(archive_msg))
         elif self.config.select_mode == "no":
             change_msg = getString("(will be left as is)")
-        if self.config.dry_run:
-            simulation_msg = getString(" dry run")
-        else:
-            simulation_msg = ""
         # print differences
         print(self.colourString(getString("Source Only (will be copied to dest): %s") %(len(source_only)), "HEADER"))
         self.log.append([getString("### SOURCE ONLY ###")])
@@ -907,6 +898,81 @@ class BackupManager:
             print(self.colourString(getString("Moved Files (will move files on dest to match source): %s") %(len(moved)), "HEADER"))
             self.log.append([getString("### MOVED FILES ###")])
             self.printMovedFiles(moved, source_dict, dest_dict)
+
+    def _performBackup(self, source_only: list, dest_only: list, changed: list, moved: list, simulation_msg: str) -> None:
+        # get databases
+        source_dict, _, _, _, _, _ = self.source.getDicts()
+        dest_dict, _, _, _, _, _ = self.dest.getDicts()
+        # perform the backup/mirror/sync
+        self.log.append([getString("### START ") + self.config.main_mode.upper() + simulation_msg.upper() + " ###"])
+        print(self.colourString(getString("Starting ") + self.config.main_mode, "HEADER"))
+        if self.config.main_mode == "mirror":
+            self.copyFiles(self.config.source, self.config.dest, source_only, source_only)
+            self.handleDeletedFiles(self.config.dest, dest_only)
+            self.handleMovedFiles(moved)
+            self.handleChangedFiles(self.config.source, self.config.dest, source_dict, dest_dict, changed)
+        elif self.config.main_mode == "backup":
+            self.copyFiles(self.config.source, self.config.dest, source_only, source_only)
+            self.handleMovedFiles(moved)
+            self.handleChangedFiles(self.config.source, self.config.dest, source_dict, dest_dict, changed)
+        elif self.config.main_mode == "sync":
+            self.copyFiles(self.config.source, self.config.dest, source_only, source_only)
+            self.copyFiles(self.config.dest, self.config.source, dest_only, dest_only)
+            self.handleMovedFiles(moved)
+            self.handleChangedFiles(self.config.source, self.config.dest, source_dict, dest_dict, changed)
+
+    ######################################
+    ### Main backup/mirror/sync method ###
+    ######################################
+
+    def backup(self):
+        """Main method, use this to run your job no matter the configuration"""
+        # dry run confirmation message
+        if self.config.dry_run:
+            simulation_msg = getString(" dry run")
+            print(self.colourString(getString("Dry Run"), "HEADER"))
+        else:
+            simulation_msg = ""
+        # init dir scanning and load previous scan data if available
+        self.source = DirInfo(self.config.source, self.config.compare_mode, self.config.config_dir,
+                              [self.config.archive_dir, self.config.log_dir, self.config.trash_dir],
+                              self.gui, self.config.force_posix_path_sep)
+        self.dest = DirInfo(self.config.dest, self.config.compare_mode, self.config.config_dir,
+                            [self.config.archive_dir, self.config.log_dir, self.config.trash_dir],
+                            self.gui, self.config.force_posix_path_sep)
+        dest_database_load_success = False
+        self.source.loadJson()
+        self.dest.loadJson()
+        if self.dest.dict_prev != {}:
+            dest_database_load_success = True
+        # scan directories (also calculates CRC if enabled) (didn't parallelize scans to prevent excess vibration of adjacent consumer grade disks and keep status bars simple)
+        self.colourPrint(getString("Scanning files on source:\n%s") %(self.config.source), "OKBLUE")
+        self.source.scanDir(self.config.stdout_status_bar)
+        if self.config.source != self.config.dest:
+            self.colourPrint(getString("Scanning files on destination:\n%s") %(self.config.dest), "OKBLUE")
+            self.dest.scanDir(self.config.stdout_status_bar)
+        else:
+            self.dest = self.source
+        # compare directories (should be relatively fast, all the read operations are done during scan)
+        if not self.config.scan_only:
+            self.colourPrint(getString("Comparing directories..."), "OKBLUE")
+            source_only, dest_only, changed, moved = self.source.dirCompare(self.dest,
+                                                                            self.config.nomoves,
+                                                                            self.config.filter_include_list,
+                                                                            self.config.filter_exclude_list)
+        # check for database conflicts or corruption
+        detected_database_conflicts_or_corruption = self._databaseAndCorruptionCheck(dest_database_load_success)
+        if self.config.quit_on_db_conflict and detected_database_conflicts_or_corruption:
+            return self.abortRun()
+        # print differences between current and previous scans then exit if only scanning
+        if self.config.scan_only:
+            self._printAndLogScanOnlyDiffSummary()
+            self.log.append([getString("### SCAN COMPLETED ###")])
+            self.writeLog("database.json")
+            print(self.colourString(getString("Completed!"), "OKGREEN"))
+            return 0
+        # print differences between source and dest
+        self._printAndLogCompareDiffSummary(source_only, dest_only, changed, moved)
         # exit if directories already match
         if len(source_only) == 0 and len(dest_only) == 0 and len(changed) == 0 and len(moved) == 0:
             print(self.colourString(getString("Directories already match, completed!"), "OKGREEN"))
@@ -924,22 +990,7 @@ class BackupManager:
             if len(go) == 0 or go[0].lower() != "y":
                 return self.abortRun()
         # backup operations
-        self.log.append([getString("### START ") + self.config.main_mode.upper() + simulation_msg.upper() + " ###"])
-        print(self.colourString(getString("Starting ") + self.config.main_mode, "HEADER"))
-        if self.config.main_mode == "mirror":
-            self.copyFiles(self.config.source, self.config.dest, source_only, source_only)
-            self.handleDeletedFiles(self.config.dest, dest_only)
-            self.handleMovedFiles(moved)
-            self.handleChangedFiles(self.config.source, self.config.dest, source_dict, dest_dict, changed)
-        elif self.config.main_mode == "backup":
-            self.copyFiles(self.config.source, self.config.dest, source_only, source_only)
-            self.handleMovedFiles(moved)
-            self.handleChangedFiles(self.config.source, self.config.dest, source_dict, dest_dict, changed)
-        elif self.config.main_mode == "sync":
-            self.copyFiles(self.config.source, self.config.dest, source_only, source_only)
-            self.copyFiles(self.config.dest, self.config.source, dest_only, dest_only)
-            self.handleMovedFiles(moved)
-            self.handleChangedFiles(self.config.source, self.config.dest, source_dict, dest_dict, changed)
+        self._performBackup(source_only, dest_only, changed, moved, simulation_msg)
         self.log.append([getString("### COMPLETED ###")])
         self.writeLog("database.json")
         print(self.colourString(getString("Completed!"), "OKGREEN"))
